@@ -91,9 +91,6 @@ Application::Application(int& argc, char** argv):
     userDirsWatcher_(nullptr),
     lxqtRunning_(false) {
 
-    // mainly used to disable tab DND outside X11 because Wayland has problem with QDrag
-    isX11_ = QGuiApplication::platformName() == QStringLiteral("xcb");
-
     argc_ = argc;
     argv_ = argv;
 
@@ -523,6 +520,8 @@ void Application::onFindFileAccepted() {
     settings_.setSearchContentRegexp(dlg->contentRegexp());
     settings_.setSearchRecursive(dlg->recursive());
     settings_.setSearchhHidden(dlg->searchhHidden());
+    settings_.addNamePattern(dlg->namePattern());
+    settings_.addContentPattern(dlg->contentPattern());
 
     Fm::FilePathList paths;
     paths.emplace_back(dlg->searchUri());
@@ -551,6 +550,8 @@ void Application::findFiles(QStringList paths) {
     dlg->setContentRegexp(settings_.searchContentRegexp());
     dlg->setRecursive(settings_.searchRecursive());
     dlg->setSearchhHidden(settings_.searchhHidden());
+    dlg->addNamePatterns(settings_.namePatterns());
+    dlg->addContentPatterns(settings_.contentPatterns());
 
     dlg->show();
 }
