@@ -72,10 +72,6 @@ public:
         return libFm_;
     }
 
-    bool isX11() const {
-      return isX11_;
-    }
-
     // public interface exported via dbus
     void launchFiles(const QString& cwd, const QStringList& paths, bool inNewWindow, bool reopenLastTabs);
     void setWallpaper(const QString& path, const QString& modeString);
@@ -158,8 +154,6 @@ private:
     QString userDesktopFolder_;
     bool lxqtRunning_;
 
-    bool isX11_;
-
     int argc_;
     char** argv_;
 };
