@@ -27,7 +27,6 @@
 #include <QFile>
 #include <QDir>
 #include <QSaveFile>
-#include <QRegExp>
 #include <QDebug>
 #include <QStandardPaths>
 #include <libfm-qt/filedialog.h>
@@ -129,6 +128,11 @@ DesktopPreferencesDialog::DesktopPreferencesDialog(QWidget* parent, Qt::WindowFl
   ui.vMargin->setValue(settings.desktopCellMargins().height());
   connect(ui.lockMargins, &QAbstractButton::clicked, this, &DesktopPreferencesDialog::lockMargins);
 
+  ui.leftMargin->setValue(settings.workAreaMargins().left());
+  ui.topMargin->setValue(settings.workAreaMargins().top());
+  ui.rightMargin->setValue(settings.workAreaMargins().right());
+  ui.bottomMargin->setValue(settings.workAreaMargins().bottom());
+
   ui.defaultFileManager->setChecked(settings.openWithDefaultFileManager());
 
   ui.allSticky->setChecked(settings.allSticky());
@@ -208,6 +212,11 @@ void DesktopPreferencesDialog::applySettings()
 
   settings.setDesktopCellMargins(QSize(ui.hMargin->value(), ui.vMargin->value()));
 
+  settings.setWorkAreaMargins(QMargins(ui.leftMargin->value(),
+                                       ui.topMargin->value(),
+                                       ui.rightMargin->value(),
+                                       ui.bottomMargin->value()));
+
   settings.setOpenWithDefaultFileManager(ui.defaultFileManager->isChecked());
 
   settings.setAllSticky(ui.allSticky->isChecked());
