@@ -63,7 +63,6 @@
 #include <xcb/xcb.h>
 #include <X11/Xlib.h>
 
-#define WORK_AREA_MARGIN 12 // margin of the work area
 #define MIN_SLIDE_INTERVAL 5*60000 // 5 min
 #define MAX_SLIDE_INTERVAL (24*60+55)*60000 // 24 h and 55 min
 
@@ -128,6 +127,7 @@ DesktopWindow::DesktopWindow(int screenNum):
         proxyModel_ = new Fm::ProxyFolderModel();
         proxyModel_->setSourceModel(model_);
         proxyModel_->setShowThumbnails(settings.showThumbnails());
+        proxyModel_->setBackupAsHidden(settings.backupAsHidden());
         proxyModel_->sort(settings.desktopSortColumn(), settings.desktopSortOrder());
         proxyModel_->setFolderFirst(settings.desktopSortFolderFirst());
         proxyModel_->setHiddenLast(settings.desktopSortHiddenLast());
@@ -139,6 +139,7 @@ DesktopWindow::DesktopWindow(int screenNum):
         connect(proxyModel_, &Fm::ProxyFolderModel::sortFilterChanged, this, &DesktopWindow::onModelSortFilterChanged);
 
         connect(this, &Fm::FolderView::inlineRenamed, this, &DesktopWindow::onInlineRenaming);
+        connect(this, &Fm::FolderView::dropIsDecided, this, &DesktopWindow::onDecidingDrop);
     }
 
     // remove frame
@@ -173,7 +174,7 @@ DesktopWindow::DesktopWindow(int screenNum):
     shortcut = new QShortcut(QKeySequence(Qt::Key_F2), this); // rename
     connect(shortcut, &QShortcut::activated, this, &DesktopWindow::onRenameActivated);
 
-    shortcut = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_F2), this); // bulk rename
+    shortcut = new QShortcut(QKeySequence(Qt::SHIFT + Qt::Key_F2), this); // bulk rename
     connect(shortcut, &QShortcut::activated, this, &DesktopWindow::onBulkRenameActivated);
 
     shortcut = new QShortcut(QKeySequence(Qt::ALT + Qt::Key_Return), this); // properties
@@ -644,88 +645,107 @@ QImage DesktopWindow::loadWallpaperFile(QSize requiredSize) {
 // really generate the background pixmap according to current settings and apply it.
 void DesktopWindow::updateWallpaper() {
     if(wallpaperMode_ != WallpaperNone) {  // use wallpaper
+        auto screen = getDesktopScreen();
+        if(screen == nullptr) {
+            return;
+        }
         QPixmap pixmap;
         QImage image;
         Settings& settings = static_cast<Application* >(qApp)->settings();
-        auto screen = getDesktopScreen();
-        bool perScreenWallpaper(screen != nullptr && screen->virtualSiblings().size() > 1 && settings.perScreenWallpaper());
+        const auto screens = screen->virtualSiblings();
+        bool perScreenWallpaper(screens.size() > 1 && settings.perScreenWallpaper());
+
+        // the pixmap's size should be calculated by considering
+        // the positions and device pixel ratios of all screens
+        QRect pixmapRect;
+        for(const auto& scr : screens) {
+            pixmapRect |= QRect(scr->geometry().topLeft(), scr->size() * scr->devicePixelRatio());
+        }
+        const QSize pixmapSize = pixmapRect.size();
+
+        // the pixmap's device pixel ratio
+        qreal DPRatio = windowHandle() ? windowHandle()->devicePixelRatio() : qApp->devicePixelRatio();
+
         if(wallpaperMode_ == WallpaperTile) { // use the original size
             image = getWallpaperImage();
             if(!image.isNull()) {
                 // Note: We can't use the QPainter::drawTiledPixmap(), because it doesn't tile
                 // correctly for background pixmaps bigger than the current screen size.
-                const QSize s = size();
-                pixmap = QPixmap{s};
+                pixmap = QPixmap{pixmapSize};
                 QPainter painter{&pixmap};
-                for (int x = 0; x < s.width(); x += image.width()) {
-                    for (int y = 0; y < s.height(); y += image.height()) {
+                for (int x = 0; x < pixmapSize.width(); x += image.width()) {
+                    for (int y = 0; y < pixmapSize.height(); y += image.height()) {
                         painter.drawImage(x, y, image);
                     }
                 }
+                pixmap.setDevicePixelRatio(DPRatio);
             }
         }
         else if(wallpaperMode_ == WallpaperStretch) {
             if(perScreenWallpaper) {
-                const QSize s = size();
-                pixmap = QPixmap{s};
+                pixmap = QPixmap{pixmapSize};
                 QPainter painter{&pixmap};
                 pixmap.fill(bgColor_);
                 image = getWallpaperImage();
                 if(!image.isNull()) {
                     QImage scaled;
-                    const auto screens = screen->virtualSiblings();
                     for(const auto& scr : screens) {
-                        scaled = image.scaled(scr->size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+                        scaled = image.scaled(scr->size() * scr->devicePixelRatio(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
                         painter.drawImage(scr->geometry().x(), scr->geometry().y(), scaled);
                     }
                 }
+                pixmap.setDevicePixelRatio(DPRatio);
             }
             else {
-                image = loadWallpaperFile(size());
+                image = loadWallpaperFile(pixmapSize);
                 pixmap = QPixmap::fromImage(image);
+                pixmap.setDevicePixelRatio(DPRatio);
             }
         }
         else { // WallpaperCenter || WallpaperFit
             if(perScreenWallpaper) {
-                const QSize s = size();
-                pixmap = QPixmap{s};
+                pixmap = QPixmap{pixmapSize};
                 QPainter painter{&pixmap};
                 pixmap.fill(bgColor_);
                 image = getWallpaperImage();
                 if(!image.isNull()) {
                     QImage scaled;
                     int x, y;
-                    const auto screens = screen->virtualSiblings();
                     if(wallpaperMode_ == WallpaperCenter) {
                         for(const auto& scr : screens) {
+                            const auto scrSize = scr->size() * scr->devicePixelRatio();
                             // get the gap between image and screen to avoid overlapping and displacement
-                            int x_gap = (image.width() - scr->geometry().width()) / 2;
-                            int y_gap = (image.height() - scr->geometry().height()) / 2;
-                            scaled = image.copy(qMax(x_gap, 0), qMax(y_gap, 0), scr->geometry().width(), scr->geometry().height());
+                            int x_gap = (image.width() - scrSize.width()) / 2;
+                            int y_gap = (image.height() - scrSize.height()) / 2;
+                            scaled = image.copy(qMax(x_gap, 0), qMax(y_gap, 0), scrSize.width(), scrSize.height());
                             x = scr->geometry().x() + qMax(0, -x_gap);
-                            y = scr->geometry().y() + qMax(0, - y_gap);
+                            y = scr->geometry().y() + qMax(0, -y_gap);
+                            painter.save();
+                            painter.setClipRect(QRect(x, y, image.width(), image.height()));
                             painter.drawImage(x, y, scaled);
+                            painter.restore();
                         }
                     }
                     else if((wallpaperMode_ == WallpaperFit || wallpaperMode_ == WallpaperZoom)
                             && image.width() > 0 && image.height() > 0) {
                         for(const auto& scr : screens) {
+                            const auto scrSize = scr->size() * scr->devicePixelRatio();
                             // get the screen-to-image ratio to calculate the scale factors
-                            const qreal w_ratio = static_cast<qreal>(scr->geometry().width()) / image.width();
-                            const qreal h_ratio = static_cast<qreal>(scr->geometry().height()) / image.height();
+                            const qreal w_ratio = static_cast<qreal>(scrSize.width()) / image.width();
+                            const qreal h_ratio = static_cast<qreal>(scrSize.height()) / image.height();
                             if(w_ratio <= h_ratio) {
                                 if(wallpaperMode_ == WallpaperFit) {
                                     // fit horizontally
-                                    scaled = image.scaledToWidth(scr->geometry().width(), Qt::SmoothTransformation);
+                                    scaled = image.scaledToWidth(scrSize.width(), Qt::SmoothTransformation);
                                     x = scr->geometry().x();
-                                    y = scr->geometry().y() + (scr->geometry().height() - scaled.height()) / 2;
+                                    y = scr->geometry().y() + (scrSize.height() - scaled.height()) / 2;
                                 }
                                 else { // zoom
                                     // fit vertically
-                                    scaled = image.scaledToHeight(scr->geometry().height(), Qt::SmoothTransformation);
+                                    scaled = image.scaledToHeight(scrSize.height(), Qt::SmoothTransformation);
                                     // crop to avoid overlapping
-                                    int x_gap = (scaled.width() - scr->geometry().width()) / 2;
-                                    scaled = scaled.copy(x_gap, 0, scr->geometry().width(), scaled.height());
+                                    int x_gap = (scaled.width() - scrSize.width()) / 2;
+                                    scaled = scaled.copy(x_gap, 0, scrSize.width(), scaled.height());
                                     x = scr->geometry().x();
                                     y = scr->geometry().y();
                                 }
@@ -733,16 +753,16 @@ void DesktopWindow::updateWallpaper() {
                             else  { // w_ratio > h_ratio
                                 if(wallpaperMode_ == WallpaperFit) {
                                     // fit vertically
-                                    scaled = image.scaledToHeight(scr->geometry().height(), Qt::SmoothTransformation);
-                                    x = scr->geometry().x() + (scr->geometry().width() - scaled.width()) / 2;
+                                    scaled = image.scaledToHeight(scrSize.height(), Qt::SmoothTransformation);
+                                    x = scr->geometry().x() + (scrSize.width() - scaled.width()) / 2;
                                     y = scr->geometry().y();
                                 }
                                 else { // zoom
                                     // fit horizonatally
-                                    scaled = image.scaledToWidth(scr->geometry().width(), Qt::SmoothTransformation);
+                                    scaled = image.scaledToWidth(scrSize.width(), Qt::SmoothTransformation);
                                     // crop to avoid overlapping
-                                    int y_gap = (scaled.height() - scr->geometry().height()) / 2;
-                                    scaled = scaled.copy(0, y_gap, scaled.width(), scr->geometry().height());
+                                    int y_gap = (scaled.height() - scrSize.height()) / 2;
+                                    scaled = scaled.copy(0, y_gap, scaled.width(), scrSize.height());
                                     x = scr->geometry().x();
                                     y = scr->geometry().y();
                                 }
@@ -751,6 +771,7 @@ void DesktopWindow::updateWallpaper() {
                         }
                     }
                 }
+                pixmap.setDevicePixelRatio(DPRatio);
             }
             else {
                 if(wallpaperMode_ == WallpaperCenter) {
@@ -770,17 +791,18 @@ void DesktopWindow::updateWallpaper() {
                     if(origSize.isValid()) {
                         QSize desiredSize = origSize;
                         Qt::AspectRatioMode mode = (wallpaperMode_ == WallpaperFit ? Qt::KeepAspectRatio : Qt::KeepAspectRatioByExpanding);
-                        desiredSize.scale(width(), height(), mode);
+                        desiredSize.scale(pixmapSize, mode);
                         image = loadWallpaperFile(desiredSize); // load the scaled image
                     }
                 }
                 if(!image.isNull()) {
-                    pixmap = QPixmap(size());
+                    pixmap = QPixmap{pixmapSize};
                     QPainter painter(&pixmap);
                     pixmap.fill(bgColor_);
-                    int x = (width() - image.width()) / 2;
-                    int y = (height() - image.height()) / 2;
+                    int x = (pixmapSize.width() - image.width()) / 2;
+                    int y = (pixmapSize.height() - image.height()) / 2;
                     painter.drawImage(x, y, image);
+                    pixmap.setDevicePixelRatio(DPRatio);
                 }
             }
         }
@@ -868,6 +890,14 @@ void DesktopWindow::nextWallpaper() {
 }
 
 void DesktopWindow::updateFromSettings(Settings& settings, bool changeSlide) {
+    // geneeral PCManFM::View settings
+    setAutoSelectionDelay(settings.singleClick() ? settings.autoSelectionDelay() : 0);
+    setCtrlRightClick(settings.ctrlRightClick());
+    if(proxyModel_) {
+        proxyModel_->setShowThumbnails(settings.showThumbnails());
+        proxyModel_->setBackupAsHidden(settings.backupAsHidden());
+    }
+
     setDesktopFolder();
     setWallpaperFile(settings.wallpaper());
     setWallpaperMode(settings.wallpaperMode());
@@ -1076,7 +1106,7 @@ void DesktopWindow::selectAll() {
 void DesktopWindow::onCreatingShortcut() {
     DesktopEntryDialog* dlg = new DesktopEntryDialog(this);
     dlg->setAttribute(Qt::WA_DeleteOnClose);
-    connect(dlg, &DesktopEntryDialog::desktopEntryCreated, [this] (const QString& name) {
+    connect(dlg, &DesktopEntryDialog::desktopEntryCreated, [this] (const Fm::FilePath&, const QString& name) {
         filesToTrust_ << name;
     });
     dlg->show();
@@ -1182,8 +1212,13 @@ void DesktopWindow::removeBottomGap() {
     auto itemSize = delegate->itemSize();
     //qDebug() << "delegate:" << delegate->itemSize();
     QSize cellMargins = getMargins();
+    Settings& settings = static_cast<Application* >(qApp)->settings();
     int workAreaHeight = screen->availableVirtualGeometry().height()
-                         - 2 * WORK_AREA_MARGIN;
+                         - settings.workAreaMargins().top()
+                         - settings.workAreaMargins().bottom();
+    if(workAreaHeight <= 0) {
+        return;
+    }
     int cellHeight = itemSize.height() + listView_->spacing();
     int iconNumber = workAreaHeight / cellHeight;
     int bottomGap = workAreaHeight % cellHeight;
@@ -1198,7 +1233,6 @@ void DesktopWindow::removeBottomGap() {
     qreal exactNumber = (static_cast<qreal>(cellHeight) - static_cast<qreal>(bottomGap))
                         / (2 * static_cast<qreal>(iconNumber) + static_cast<qreal>(2));
     int subtrahend = (int)exactNumber + ((int)exactNumber == exactNumber ? 0 : 1);
-    Settings& settings = static_cast<Application*>(qApp)->settings();
     int minCellHeight = settings.desktopCellMargins().height();
     if(subtrahend > 0
             && cellMargins.height() - subtrahend >= minCellHeight) {
@@ -1239,7 +1273,9 @@ void DesktopWindow::paintBackground(QPaintEvent* event) {
         painter.fillRect(event->rect(), QBrush(bgColor_));
     }
     else {
-        painter.drawPixmap(event->rect(), wallpaperPixmap_, event->rect());
+        QRectF r(QPointF(event->rect().topLeft()) * wallpaperPixmap_.devicePixelRatio(),
+                 QSizeF(event->rect().size()) * wallpaperPixmap_.devicePixelRatio());
+        painter.drawPixmap(event->rect(), wallpaperPixmap_, r.toRect());
     }
 }
 
@@ -1278,11 +1314,18 @@ void DesktopWindow::trustOurDesktopShortcut(std::shared_ptr<const Fm::FileInfo>
 
 QRect DesktopWindow::getWorkArea(QScreen* screen) const {
     QRect workArea = screen->availableVirtualGeometry();
-    workArea.adjust(WORK_AREA_MARGIN, WORK_AREA_MARGIN, -WORK_AREA_MARGIN, -WORK_AREA_MARGIN);
+    QMargins margins = static_cast<Application* >(qApp)->settings().workAreaMargins();
     // switch between right and left with RTL to use the usual (LTR) calculations later
     if(layoutDirection() == Qt::RightToLeft) {
+        int right = margins.right();
+        margins.setRight(margins.left());
+        margins.setLeft(right);
+        workArea = workArea.marginsRemoved(margins);
         workArea.moveLeft(rect().right() - workArea.right());
     }
+    else {
+        workArea = workArea.marginsRemoved(margins);
+    }
     return workArea;
 }
 
@@ -1686,6 +1729,9 @@ QModelIndex DesktopWindow::navigateWithKey(int key, Qt::KeyboardModifiers modifi
         QRect workArea = getWorkArea(screen);
         int columns = workArea.width() / (itemSize.width() + listView_->spacing());
         int rows = workArea.height() / (itemSize.height() + listView_->spacing());
+        if(columns <= 0 || rows <= 0) {
+            break;
+        }
         bool rtl(layoutDirection() == Qt::RightToLeft);
         while(!index.isValid() && workArea.contains(pos)) {
             switch(key) {
@@ -1948,7 +1994,7 @@ void DesktopWindow::childDropEvent(QDropEvent* e) {
         // move selected items to the drop position, preserving their relative positions
         QPoint dropPos = e->pos();
 
-        if(curIndx.isValid()) {
+        if(curIndx.isValid() && !workArea.isEmpty()) {
             QPoint curPoint = listView_->visualRect(curIndx).topLeft();
 
             // first move the current item to the drop position
@@ -2013,6 +2059,9 @@ void DesktopWindow::childDropEvent(QDropEvent* e) {
             }
         }
 
+        // store current positions before the drop; see DesktopWindow::onDecidingDrop()
+        storeCustomPos();
+
         Fm::FolderView::childDropEvent(e);
 
         // remove the drop indicator after the drop is finished
@@ -2026,11 +2075,9 @@ void DesktopWindow::childDropEvent(QDropEvent* e) {
             }
         }
 
-        // position dropped items successively, starting with the drop rectangle
-        if(mimeData->hasUrls()
-           && (e->dropAction() == Qt::CopyAction
-               || e->dropAction() == Qt::MoveAction
-               || e->dropAction() == Qt::LinkAction)) {
+        // reserve successive positions for dropped items, starting with the drop rectangle
+        if(!workArea.isEmpty()
+           && mimeData->hasUrls()) {
             const QString desktopDir = XdgDir::readDesktopDir() + QString(QLatin1String("/"));
             QPoint dropPos = e->pos();
             if(rtl) { // see the previous case for the reason
@@ -2046,16 +2093,31 @@ void DesktopWindow::childDropEvent(QDropEvent* e) {
                     reachedLastCell = stickToPosition(name.toStdString(), dropPos, workArea, grid, reachedLastCell);
                 }
             }
-            storeCustomPos();
+            // wait for FolderView::dropIsDecided() to know whether the new positions should be stored
+            // on accepting the drop or the original positions should be restored on cancelling it
         }
     }
 }
 
+void DesktopWindow::onDecidingDrop(bool accepted) {
+    if(accepted) {
+        storeCustomPos();
+    }
+    else {
+        customItemPos_.clear();
+        customItemPos_ = customPosStorage_;
+    }
+}
+
 // NOTE: This function positions items from top to bottom and left to right,
 // starting from the drop point, and carries the existing sticky items with them,
 // until it reaches the last cell and then puts the remaining items in the opposite
 // direction. In this way, it creates a natural DND, especially with multiple files.
 bool DesktopWindow::stickToPosition(const std::string& file, QPoint& pos, const QRect& workArea, const QSize& grid, bool reachedLastCell) {
+    if(workArea.isEmpty()) {
+        return reachedLastCell;
+    }
+
     // normalize the position, depending on the positioning direction
     if(!reachedLastCell) { // default direction: top -> bottom, left -> right
 
@@ -2155,6 +2217,10 @@ void DesktopWindow::alignToGrid(QPoint& pos, const QPoint& topLeft, const QSize&
 // the text or icon rectangle but we make an exception for Trash because we want
 // to trash dropped items once the drop point is inside the Trash cell.
 QModelIndex DesktopWindow::indexForPos(bool* isTrash, const QPoint& pos, const QRect& workArea, const QSize& grid) const {
+    if(workArea.isEmpty()) {
+        return QModelIndex();
+    }
+
     // first normalize the position
     QPoint p(pos);
 
