@@ -481,9 +481,7 @@ void MainWindow::addViewFrame(const Fm::FilePath& path) {
     Application* app = static_cast<Application*>(qApp);
     Settings& settings = app->settings();
     ViewFrame* viewFrame = new ViewFrame();
-    // No tab DND with the split view.
-    // WARNING: Wayland has a serious issue related to QDrag that can result in a crash.
-    viewFrame->getTabBar()->setDetachable(!splitView_ && app->isX11());
+    viewFrame->getTabBar()->setDetachable(!splitView_); // no tab DND with the split view
     viewFrame->getTabBar()->setTabsClosable(settings.showTabClose());
     ui.viewSplitter->addWidget(viewFrame); // the splitter takes ownership of viewFrame
     if(ui.viewSplitter->count() == 1) {
@@ -509,8 +507,9 @@ void MainWindow::addViewFrame(const Fm::FilePath& path) {
             on_actionNewTab_triggered();
         }
     });
-    connect(viewFrame->getTabBar(), &TabBar::tabDetached, this, &MainWindow::detachTab);
     connect(viewFrame->getStackedWidget(), &QStackedWidget::widgetRemoved, this, &MainWindow::onStackedWidgetWidgetRemoved);
+    // the tab will be detached only after the DND is finished
+    connect(viewFrame->getTabBar(), &TabBar::tabDetached, this, &MainWindow::detachTab, Qt::QueuedConnection);
 
     if(path) {
         addTab(path, viewFrame);
@@ -579,7 +578,7 @@ void MainWindow::on_actionSplitView_triggered(bool checked) {
         }
 
         // enable tab DND
-        activeViewFrame_->getTabBar()->setDetachable(app->isX11());
+        activeViewFrame_->getTabBar()->setDetachable(true);
         setAcceptDrops(true);
 
         activeViewFrame_->removeTopBar();
@@ -973,7 +972,27 @@ void MainWindow::on_actionShowHidden_triggered(bool checked) {
 }
 
 void MainWindow::on_actionShowThumbnails_triggered(bool checked) {
-    currentPage()->setShowThumbnails(checked);
+    QWidgetList windows = qApp->topLevelWidgets();
+    QWidgetList::iterator it;
+    for(it = windows.begin(); it != windows.end(); ++it) {
+        QWidget* window = *it;
+        if(window->inherits("PCManFM::MainWindow")) {
+            MainWindow* mainWindow = static_cast<MainWindow*>(window);
+            mainWindow->ui.actionShowThumbnails->setChecked(checked); // doesn't call this function
+            for(int i = 0; i < mainWindow->ui.viewSplitter->count(); ++i) {
+                if(ViewFrame* viewFrame = qobject_cast<ViewFrame*>(mainWindow->ui.viewSplitter->widget(i))) {
+                    int n = viewFrame->getStackedWidget()->count();
+                    for(int j = 0; j < n; ++j) {
+                        if(TabPage* page = static_cast<TabPage*>(viewFrame->getStackedWidget()->widget(j))) {
+                            page->setShowThumbnails(checked);
+                        }
+                    }
+                }
+            }
+        }
+    }
+    // this setting is shared by Desktop too
+    static_cast<Application*>(qApp)->updateDesktopsFromSettings(false);
 }
 
 void MainWindow::on_actionByFileName_triggered(bool /*checked*/) {
@@ -1057,7 +1076,7 @@ void MainWindow::on_actionFilter_triggered(bool checked) {
     static_cast<Application*>(qApp)->settings().setShowFilter(checked);
     // show/hide filter-bars and disable/enable their transience for all tabs
     // (of all view frames) in all windows because this is a global setting
-    QWidgetList windows = static_cast<Application*>(qApp)->topLevelWidgets();
+    QWidgetList windows = qApp->topLevelWidgets();
     QWidgetList::iterator it;
     for(it = windows.begin(); it != windows.end(); ++it) {
         QWidget* window = *it;
@@ -1851,6 +1870,10 @@ void MainWindow::on_actionSelectAll_triggered() {
     currentPage()->selectAll();
 }
 
+void MainWindow::on_actionDeselectAll_triggered() {
+    currentPage()->deselectAll();
+}
+
 void MainWindow::on_actionInvertSelection_triggered() {
     currentPage()->invertSelection();
 }
@@ -1905,7 +1928,7 @@ void MainWindow::onBackForwardContextMenu(QPoint pos) {
     TabPage* page = currentPage();
     Fm::BrowseHistory& history = page->browseHistory();
     int current = history.currentIndex();
-    QMenu menu;
+    QMenu menu(this);
     for(size_t i = 0; i < history.size(); ++i) {
         const BrowseHistoryItem& item = history.at(i);
         auto path = item.path();
@@ -1952,7 +1975,7 @@ void MainWindow::tabContextMenu(const QPoint& pos) {
             return;
         }
 
-        QMenu menu;
+        QMenu menu(this);
 
         // tab closing actions
         if(rightClickIndex_ > 0) {
@@ -2035,7 +2058,14 @@ void MainWindow::dragEnterEvent(QDragEnterEvent* event) {
 
 void MainWindow::dropEvent(QDropEvent* event) {
     if(event->mimeData()->hasFormat(QStringLiteral("application/pcmanfm-qt-tab"))) {
-        dropTab(event->source());
+        if(QObject *sourseObject = event->source()) {
+            // announce that the tab drop is accepted by us (see TabBar::mouseMoveEvent)
+            sourseObject->setProperty(TabBar::tabDropped, true);
+            // the tab will be dropped (moved) after the DND is finished
+            QTimer::singleShot(0, sourseObject, [this, sourseObject]() {
+                dropTab(sourseObject);
+            });
+        }
     }
     event->acceptProposedAction();
 }
