@@ -34,12 +34,12 @@ class PreferencesDialog : public QDialog {
 
 public:
     // activePage is the name of page to select (general, display, advanced...)
-    explicit PreferencesDialog(QString activePage = QString(), QWidget* parent = 0);
+    explicit PreferencesDialog(const QString& activePage = QString(), QWidget* parent = nullptr);
     virtual ~PreferencesDialog();
 
     virtual void accept();
 
-    void selectPage(QString name);
+    void selectPage(const QString& name);
 
 protected Q_SLOTS:
     void lockMargins(bool lock);
