@@ -134,6 +134,7 @@ Settings::Settings():
     templateRunApp_(false),
     folderViewCellMargins_(QSize(3, 3)),
     desktopCellMargins_(QSize(3, 1)),
+    workAreaMargins_(QMargins(12, 12, 12, 12)),
     openWithDefaultFileManager_(false),
     allSticky_(false),
     searchNameCaseInsensitive_(false),
@@ -142,6 +143,7 @@ Settings::Settings():
     searchContentRegexp_(true),
     searchRecursive_(false),
     searchhHidden_(false),
+    maxSearchHistory_(0),
     recentFilesNumber_(0) {
 }
 
@@ -268,6 +270,14 @@ bool Settings::loadFile(QString filePath) {
 
     desktopCellMargins_ = (settings.value(QStringLiteral("DesktopCellMargins"), QSize(3, 1)).toSize()
                            .expandedTo(QSize(0, 0))).boundedTo(QSize(48, 48));
+    auto l = settings.value(QStringLiteral("WorkAreaMargins")).toList();
+    if(l.size() >= 4) {
+        workAreaMargins_.setLeft(qBound(0, l.at(0).toInt(), 200));
+        workAreaMargins_.setTop(qBound(0, l.at(1).toInt(), 200));
+        workAreaMargins_.setRight(qBound(0, l.at(2).toInt(), 200));
+        workAreaMargins_.setBottom(qBound(0, l.at(3).toInt(), 200));
+    }
+
     openWithDefaultFileManager_ = settings.value(QStringLiteral("OpenWithDefaultFileManager"), false).toBool();
     allSticky_ = settings.value(QStringLiteral("AllSticky"), false).toBool();
     settings.endGroup();
@@ -349,6 +359,11 @@ bool Settings::loadFile(QString filePath) {
     searchContentRegexp_ = settings.value(QStringLiteral("searchContentRegexp"), true).toBool();
     searchRecursive_ = settings.value(QStringLiteral("searchRecursive"), false).toBool();
     searchhHidden_ = settings.value(QStringLiteral("searchhHidden"), false).toBool();
+    maxSearchHistory_ = qBound(0, settings.value(QStringLiteral("MaxSearchHistory"), 0).toInt(), 50);
+    namePatterns_ = settings.value(QStringLiteral("NamePatterns")).toStringList();
+    namePatterns_.removeDuplicates();
+    contentPatterns_ = settings.value(QStringLiteral("ContentPatterns")).toStringList();
+    contentPatterns_.removeDuplicates();
     settings.endGroup();
 
     return true;
@@ -410,6 +425,11 @@ bool Settings::saveFile(QString filePath) {
     settings.setValue(QStringLiteral("SortFolderFirst"), desktopSortFolderFirst_);
     settings.setValue(QStringLiteral("SortHiddenLast"), desktopSortHiddenLast_);
     settings.setValue(QStringLiteral("DesktopCellMargins"), desktopCellMargins_);
+    QList<QVariant> l{workAreaMargins_.left(),
+                      workAreaMargins_.top(),
+                      workAreaMargins_.right(),
+                      workAreaMargins_.bottom()};
+    settings.setValue(QStringLiteral("WorkAreaMargins"), l);
     settings.setValue(QStringLiteral("OpenWithDefaultFileManager"), openWithDefaultFileManager_);
     settings.setValue(QStringLiteral("AllSticky"), allSticky_);
     settings.endGroup();
@@ -494,6 +514,9 @@ bool Settings::saveFile(QString filePath) {
     settings.setValue(QStringLiteral("searchContentRegexp"), searchContentRegexp_);
     settings.setValue(QStringLiteral("searchRecursive"), searchRecursive_);
     settings.setValue(QStringLiteral("searchhHidden"), searchhHidden_);
+    settings.setValue(QStringLiteral("MaxSearchHistory"), maxSearchHistory_);
+    settings.setValue(QStringLiteral("NamePatterns"), namePatterns_);
+    settings.setValue(QStringLiteral("ContentPatterns"), contentPatterns_);
     settings.endGroup();
 
     return true;
@@ -548,6 +571,52 @@ void Settings::saveRecentFiles() {
     settings.endGroup();
 }
 
+void Settings::clearSearchHistory() {
+    namePatterns_.clear();
+    contentPatterns_.clear();
+}
+
+void Settings::setMaxSearchHistory(int max) {
+    maxSearchHistory_ = qMax(max, 0);
+    if(maxSearchHistory_ == 0) {
+        namePatterns_.clear();
+        contentPatterns_.clear();
+    }
+    else {
+        while(namePatterns_.size() > maxSearchHistory_) {
+            namePatterns_.removeLast();
+        }
+        while(contentPatterns_.size() > maxSearchHistory_) {
+            contentPatterns_.removeLast();
+        }
+    }
+}
+
+void Settings::addNamePattern(const QString& pattern) {
+    if(maxSearchHistory_ == 0 || pattern.isEmpty()
+       // "*" is too trivial with a regex search
+       || (searchNameRegexp_ && pattern == QLatin1String("*"))) {
+        return;
+    }
+    namePatterns_.removeOne(pattern);
+    namePatterns_.prepend(pattern);
+    while(namePatterns_.size() > maxSearchHistory_) {
+        namePatterns_.removeLast();
+    }
+}
+
+void Settings::addContentPattern(const QString& pattern) {
+    if(maxSearchHistory_ == 0 || pattern.isEmpty()
+       || (searchContentRegexp_ && pattern == QLatin1String("*"))) {
+        return;
+    }
+    contentPatterns_.removeOne(pattern);
+    contentPatterns_.prepend(pattern);
+    while(contentPatterns_.size() > maxSearchHistory_) {
+        contentPatterns_.removeLast();
+    }
+}
+
 const QList<int> & Settings::iconSizes(IconType type) {
     static const QList<int> sizes_big = {96, 72, 64, 48, 32};
     static const QList<int> sizes_thumbnail = {256, 224, 192, 160, 128, 96, 64};
