@@ -788,6 +788,14 @@ public:
         desktopCellMargins_ = size;
     }
 
+    QMargins workAreaMargins() const {
+        return workAreaMargins_;
+    }
+
+    void setWorkAreaMargins(QMargins margins) {
+        workAreaMargins_ = margins;
+    }
+
     bool openWithDefaultFileManager() const {
         return openWithDefaultFileManager_;
     }
@@ -973,6 +981,26 @@ public:
         searchhHidden_ = hidden;
     }
 
+    int maxSearchHistory() const {
+        return maxSearchHistory_;
+    }
+
+    void setMaxSearchHistory(int max);
+
+    void clearSearchHistory();
+
+    QStringList namePatterns() const {
+        return namePatterns_;
+    }
+
+    void addNamePattern(const QString& pattern);
+
+    QStringList contentPatterns() const {
+        return contentPatterns_;
+    }
+
+    void addContentPattern(const QString& pattern);
+
     QList<int> getCustomColumnWidths() const {
         QList<int> l;
         for(auto width : qAsConst(customColumnWidths_)) {
@@ -1125,6 +1153,7 @@ private:
 
     QSize folderViewCellMargins_;
     QSize desktopCellMargins_;
+    QMargins workAreaMargins_;
 
     bool openWithDefaultFileManager_;
 
@@ -1137,6 +1166,9 @@ private:
     bool searchContentRegexp_;
     bool searchRecursive_;
     bool searchhHidden_;
+    int maxSearchHistory_;
+    QStringList namePatterns_;
+    QStringList contentPatterns_;
 
     // detailed list columns
     QList<QVariant> customColumnWidths_;
