@@ -438,7 +438,12 @@ void TabPage::onFilesAdded(Fm::FileInfoList files) {
 void TabPage::onFolderFinishLoading() {
     auto fi = folder_->info();
     if(fi) { // if loading of the folder fails, it's possible that we don't have FmFileInfo.
-        title_ = fi->displayName();
+        if(folder_->path().hasUriScheme("search")) {
+            title_ = tr("Search Results"); // FIXME: Localize it in libfm-qt!
+        }
+        else {
+            title_ = fi->displayName();
+        }
         Q_EMIT titleChanged();
     }
 
@@ -716,6 +721,10 @@ void TabPage::selectAll() {
     folderView_->selectAll();
 }
 
+void TabPage::deselectAll() {
+    folderView_->selectionModel()->clearSelection();
+}
+
 void TabPage::invertSelection() {
     folderView_->invertSelection();
 }
@@ -1057,11 +1066,10 @@ void TabPage::ceateShortcut() {
         if(folderPath && folderPath.isNative()) {
             DesktopEntryDialog* dlg = new DesktopEntryDialog(this, folderPath);
             dlg->setAttribute(Qt::WA_DeleteOnClose);
-            connect(dlg, &DesktopEntryDialog::desktopEntryCreated, [this] (const QString& name) {
+            connect(dlg, &DesktopEntryDialog::desktopEntryCreated, [this] (const Fm::FilePath& parent, const QString& name) {
                 // if the current directory does not have a file monitor or is changed,
                 // there will be no point to tracking the created shortcut
-                if(folder_ && folder_->hasFileMonitor()
-                   && folder_->path().isParentOf(Fm::FilePath::fromLocalPath(name.toLocal8Bit().constData()))) {
+                if(folder_ && folder_->hasFileMonitor() && folder_->path() == parent) {
                     filesToTrust_ << name;
                 }
             });
